page	,132

Title	AD.ASM Analog to Digital Converter Driver callable from Quickbasic

;************************************************************************;
;*								       	*;
;*	Copyright JK microsystems 1995	-  All Rights Reserved	  	*;
;*									*;
;*	This software is NOT shareware, freeware, or public domain.	*;
;*	It is the property of JK microsystems.				*;
;*									*;
;*	Customers of JK microsystems may modify the source code		*; 
;*	and/or distribute the binary image of this software without 	*;
;*	additional costs provided it is run only on hardware 		*;
;*	manufactured by JK microsystems.  All other use is expressly 	*;
;*	prohibited.							*;
;*									*;
;*	Update Log							*;
;*									*;
;*	Version	Date	Comments			Progammer	*;
;*	-------	-------	-------------------------------	----------	*;
;*	0.9	10-5-95	Beta test release		jds		*;
;*	1.0	3-12-98 Time constants changed for 386	jds		*;
;*									*;
;************************************************************************;

	.model medium		
	.code

;-------------------------------------------------------------------------
;
;	ADchan sets the rejection frequency of the mains power.  The only
;	two legal values are 50 and 60 decimal.  The default is 60.

	public	ADlinefreq

ADlineFreq	proc

	push 	bp			; get pointer to argument saving bp,
	mov	bp,sp			; loading it with sp 
	mov	bx,[bp+6]		; getting the offset of the parameter
	mov	ax,ss:[bx]		; and getting the parameter

	cmp	ax,60			; is it 60 hz
	jne	not_60			; no
	mov	al,76			; yes, 76 is the constant for 386
	mov	cs:[freq],al		; store it
	jmp	ADlineFreq_done		; all done

not_60:
	cmp	ax,50			; is it 50hz
	jne	ADlineFreq_done		; no, not 50 or 60, don't change
	mov	al,68			; yes, 68 is the constant 386

ADlinefreq_done:
	pop	bp			; restore bp
	ret	2			; and return

ADlinefreq	endp


;-------------------------------------------------------------------------
;
;	ADrange sets the range for the A/D conversion. Default range is 3.
;
;	0 = 0V to +5V
;	1 = +/-5V
;	2 = 0V to +10V
;	3 = +/-10V
	
	public	ADrange

ADrange	proc

	push 	bp			; get pointer to argument saving bp,
	mov	bp,sp			; loading it with sp 
	mov	bx,ss:[bp+6]		; getting the offset of the parameter
	mov	ax,ss:[bx]		; and getting the parameter

	shl	al,1			; move it to the right place 
	shl	al,1			; in the command byte
	shl	al,1
	and	al,018h			; zero all but bits 3 and 4

	mov	cs:[range],al		; store it

	pop	bp			; restore bp
	ret	2			; and return

ADrange	endp


;-------------------------------------------------------------------------
;
;	AD takes 256 samples over the period of one AC power cycle, 
;	averages them, and normalizes them so that for all ranges, the
;	value returned in volts is chan% * ( 1.25 / 2^12 ).  Note that
; 	by oversampling, we get 13-14 bits of resolution.
;
	public	AD

AD	proc	

	push 	bp			; get pointer to argument saving bp,
	mov	bp,sp			; loading it with sp 
	mov	bx,ss:[bp+6]		; getting the offset of the parameter
	mov	ax,ss:[bx]		; and getting the channel
	and	al,7			; zero all but first 3 bits
			
	or	al,cs:[range]		; OR the range into the command
	or	al,40h			; set the "internal clock" bit
	mov	bl,al			; place in bl for routine

	mov	ch,cs:[freq]		; get the freq

	mov	dx,2B0h			; A/D base I/O address

	push	di			; save di and ds
	push	ds
	mov	ax,cs			; set our ds
	mov	ds,ax
	
	mov	di,offset buffer

sample_loop:
	mov	al,bl			; get command
	out	dx,al			; send it

	mov	cl,ch			; cl determines the delay between A/D
AD_delay:				; conversions.  It is chosen to cause
	dec	cl			; 256 conversions to happen during one
	jnz	AD_delay		; AC power cycle

	in	ax,dx			; get the data

	mov	[di],ax			; store it
	inc	di			; increment di
	inc	di			; twice for word

	cmp	di,offset buffer+200h	; are we at the end of the buffer?
	jne	sample_loop		; no, back through

	mov	al,0			; buffer gets summed here, lo byte
	mov	ah,0			; mid byte
	mov	bh,0			; high byte
	mov	di,offset buffer	; point to buffer start

sum_loop:
	mov	dx,[di]			; get the word

	test	bl,8h			; are we in bipolar mode?
	jz	unipolar		; no, don't convert data
	add	dh,8h			; yes, add in offset
	and	dh,0Fh			; and zero sign to make unsigned
unipolar:
	add	ax,dx			; do the add
	jnc	math_done		; 16 bit overflow?
	inc	bh			; yes, increment hi byte

math_done:
	inc	di			; bump pointers
	inc	di			; twice for word accesses
	cmp	di,offset buffer+200h	; are we done yet?
	jne	sum_loop		; no, back through

	mov	ch,bh			; hi byte in ch
	mov	cl,ah			; mid byte in cl

; 	At this point, we have data spread over 3 bytes, 
;	
;	CH = hi byte
;	CL = mid byte
;	AL = lo byte
;
;	The hi 4 bits of CH is all 1's and the lo 4 bits of
;	AL is noise, so now we move the lower 4 bits of CH,
;	all of CL, and the upper 4 bits of al into CX.

	shl	cx,1			; shift cx to left 4 bit positions
	shl	cx,1
	shl	cx,1
	shl	cx,1

	shr	al,1			; shift al right 4 bit positions
	shr	al,1
	shr	al,1
	shr	al,1
	
	and	cx,0FFF0h		; zero the nibble where al is going
	and	al,0Fh			; zero noise nibble
	mov	ah,0			; zero hi byte so we can do word op
	or	cx,ax			; OR low nibble of al into cx

	test	bl,8h			; are we in bipolar mode?
	jz	unipolar_2		; yes, don't modify
	sub	cx,8000h		; no,convert back to signed integer
	jmp	bipolar

unipolar_2:
	shr	cx,1			; now we scale so that all ranges
	test	bl,10h			; return same volts/bit ratio
	jnz	range_done		; all done if zero to 10 range
	shr	cx,1			; shift again if zero to 5 range
	jmp	range_done

bipolar:
	test	bl,10h			; +/-10V?
	jnz	range_done		; yes, done
	sar	cx,1			; no, +/-5V so scale it

range_done:
	mov	bx,ss:[bp+6]		; get pointer into stack
	mov	ss:[bx],cx		; put the parameter in the stack

	pop	ds			; restore trashed registers
	pop	di
	pop	bp
	
	ret	2			; return with one parameter on stack

;-------------------------------------------------------------------------
;
;	Variable storage and buffer area

freq	db	60
range	db	3

buffer:
	db	200h dup(0)	
AD	endp

;-------------------------------------------------------------------------
;
;	ADone takes one sample from the selected channel at the selected gain
;
	public	ADone

ADone	proc	

	push 	bp			; get pointer to argument saving bp,
	mov	bp,sp			; loading it with sp 
	mov	bx,ss:[bp+6]		; getting the offset of the parameter
	mov	ax,ss:[bx]		; and getting the channel
	and	al,7			; zero all but first 3 bits
			
	or	al,cs:[range]		; OR the range into the command
	or	al,40h			; set the "internal clock" bit
	mov	bl,al			; place in bl for routine

	mov	ch,cs:[freq]		; get the freq

	mov	dx,2B0h			; A/D base I/O address

	mov	al,bl			; get command
	out	dx,al			; send it

	mov	cl,20			; cl determines the delay between A/D
AD_one_delay:				; conversions.  It is chosen to cause
	dec	cl			; at least a 6us delay
	jnz	AD_one_delay		; 

	in	ax,dx			; get the data
	
	shl	ax,1
	shl	ax,1
	cmp	cs:[range],0
	je	AD_one_done
	shl	ax,1
	cmp	cs:[range],3
	jne	AD_one_done
	shl	ax,1
	shl	ax,1


AD_one_done:	
	mov	bx,ss:[bp+6]		; get pointer into stack
	mov	ss:[bx],ax		; put the parameter in the stack
	pop	bp
	
	ret	2			; return with one parameter on stack

ADone	endp

	end
