
	page	58,132
Code_seg	Segment
	assume	cs:code_seg,ds:code_seg,ss:code_seg

;************************************************************************;
;*								       	*;
;*	Copyright JK microsystems 1995	-  All Rights Reserved	  	*;
;*									*;
;*	This software is NOT shareware, freeware, or public domain.	*;
;*	It is the property of JK microsystems.				*;
;*									*;
;*	Customers of JK microsystems may modify the source code		*; 
;*	and/or distribute the binary image of this software without 	*;
;*	additional costs provided it is run only on hardware 		*;
;*	manufactured by JK microsystems.  All other use is expressly 	*;
;*	prohibited.							*;
;*									*;
;*	Update Log							*;
;*									*;
;*	Version	Date	Comments			Progammer	*;
;*	-------	-------	-------------------------------	----------	*;
;*	1.0	12-4-95	Beta test release		jds		*;
;*									*;
;************************************************************************;


;************************************************************************;
;
;	Capture opens a file for output specified on the command line,
;	waits for channel 0 on the A/D converter board to exceed .25v,
;	takes 2048 samples from channel 0, converts the samples to  
; 	spreadsheet format, i.e. tab delimited decimal data, and writes 
;	the converted data to the specified file.
;
;	If you modify and assemble this file, be sure to convert it 
;	to .COM format.  If you leave it in the .EXE format, it will 
;	not be able to find the file specified on the command line in 
;	the PSP.  

	org	100h
start:
	mov	ax,cs			; make all our segments 
	mov	ds,ax			; same as cs
	mov	es,ax
	mov	ss,ax

	mov	dx,offset crlf		; print a cr and lf
	mov	ah,9h			; next we will get the command line
	int	21h			; parameter and open a file with it.
	mov	ch,0			; zero hi byte of cx
	mov	si,80h			; point to cmd line length in psp
	mov	cl,[si]			; get it in cl
	cmp	cl,0			; test for zero length filename
	jne	name_present		; jump around if non-zero length
	mov	dx,offset help		; point to help message
	mov	ah,9h			; tell dos
	int	21h			; to print it
	mov	ax,4C00h		; tell dos
	int	21h			; to terminate
name_present:
	inc	si			; point to start of string
	mov	di,offset filename	; point to buffer
	cld				; string moves increment
	rep 	movsb			; move the string into buffer
	mov	[di],byte ptr 0		; terminate string with null
	mov	di,offset filename	; point di at start of filename
skip_space:
	inc	di			; skip past first space
	cmp	[di],byte ptr ' '	; test for more leading spaces
	je	skip_space		; if another, increment past it
	mov	dx,di			; done, point dx to first char
	mov	ax,3C00h		; Create File function
	mov	cx,0			; no special attributes
	int	21h			; do it
	jnc	good_open		; jmp around if good
	mov	dx,offset open_error	; point to error message
	mov	ah,9h			; tell dos
	int	21h			; to print it
	mov	ax,4C00h		; then terminate
	int	21h
good_open:
	mov	di,offset handle	; point to handle
	mov	[di],ax			; save it
	mov	dx,offset waiting	; tell user waiting for data
	mov	ah,9h	
	int	21h

	mov	dx,AD_address		; A/D base address
	mov	bl,AD_cmd		; A/D command
	mov	di,offset raw_buffer	; buffer address
	mov	cx,800h			; number of samples

;*******************************************************************
;
;	This code waits for the A/D input to exceed 0.25v before
;	taking the run of data.  

AD_trigger_loop:
	mov	al,bl			; get command
	out	dx,al			; send it

	mov	bh,10			; wait for A/D to go done
delay_loop_1:				; change bh to change sample rate
	dec	bh			; be careful not to go faster than
	jnz	delay_loop_1		; 6us/conversion

	in	ax,dx			; get the data
	add	ah,8h			; make it unsigned by adding offset
	and	ah,0Fh			; and zeroing sign
	cmp	ax,2100			; greater than .25v
	jna	AD_trigger_loop		; no, loop
;
;	End of trigger code...
;	
;*******************************************************************

AD_loop:
	mov	al,bl			; get command
	out	dx,al			; send it

	mov	bh,10			; wait for A/D to go done
delay_loop_2:				; do not decrease below 10
	dec	bh			; increase to lower sample rate
	jnz	delay_loop_2	

	in	ax,dx			; get the data
	stosw				; store it
	loop	ad_loop
	
	mov	dx,offset converting	; tell user we're converting data
	mov	ah,9h
	int	21h

	mov	di,offset raw_buffer	; raw buffer start
	mov	si,offset ascii_buffer	; finished buffer start

convert_loop:
	mov	ax,[di]			; get value
	inc	di			; bump pointer
	inc	di
	cmp	di,offset raw_buffer_end ; test for end
	jne	not_done		; not there yet
	jmp	write_file		; done, write the file

not_done:
	add	ah,8h			; make it unsigned by adding offset
	and	ah,0Fh			; and zeroing sign

	call	bin_to_bcd		; convert it to 4 bcd digits in ax
	mov	dx,ax			; convert the bcd digits to 4 ascii
	ror	ax,1			; chars and store them
	ror	ax,1
	ror	ax,1
	ror	ax,1
	and	ah,0Fh			; make high nibble ascii
	or	ah,30h
	mov	[si],ah			; store char
	inc	si
	and	dh,0Fh			; make second highest ascii
	or	dh,30h
	mov	[si],dh
	inc	si
	and	al,0Fh			; make third highest ascii
	or	al,30h
	mov	[si],al
	inc	si
	and	dl,0Fh			; make lowest ascii
	or	dl,30h
	mov	[si],dl
	inc	si
	mov	[si],byte ptr 09h	; store a tab char
	inc	si
	mov	[si],byte ptr 0Dh	; and store a carriage return
	inc	si
	jmp	convert_loop

write_file:
	mov	dx,offset writing	; tell user we're writing the data
	mov	ah,9h
	int	21h
	mov	di,offset handle	; retrieve file handle
	mov	bx,[di]
	mov	ah,40h			; write function code
	mov	dx,offset ascii_buffer	; point to data
	mov	cx,3000h		; file length
	int	21h			; do it
	cmp	ax,3000h		; check if all bytes written
	je	good_write		; yes, return to dos
	mov	dx,offset write_error	; point to error message
	mov	ah,9h			; print it
	int	21h
good_write:
	mov	di,offset handle	; get the handle
	mov	bx,[di]
	mov	ah,3Eh			; and close the file
	int	21h			
	jnc	good_close		; terminate if ok
	mov	dx,offset close_error	; tell user close error
	mov	ah,9h
	int	21h	
good_close:
	mov	ax,4C00h		; terminate function code
	int	21h			; do it

;---------------------------------------------------------------
;
;	Convert the 12-bit binary value in ax to a 4-digit
;	BCD value
;
bin_to_bcd:
	push	bx			; make some room to work
	push	cx
	push	dx
	push	di
	mov	bx,ax			; move it to bx
	mov	ax,0			; clear ax
	mov	dx,0
	mov	cx,12			; number of bits to test
	mov	di,offset [bcd_table]	; point to table
build_value:
	shr	bx,1			; get bit
	jnc	zero_bit		; zero so do nothing
	mov	ax,[di]			; get table entry
	call	bcd_add			; add it in
zero_bit:
	inc	di			; bump pointer		
	inc	di
	loop	build_value
	mov	ax,dx
	pop	di
	pop	dx
	pop	cx
	pop	bx
	ret
	
;---------------------------------------------------------------
;
;	BCD add the contents of AX to DX
;
BCD_add:
	add	al,dl			; add low byte
	daa				; decimal adjust it
	mov	dl,al			; put it in al
	mov	al,ah			; put high byte in al
	adc	al,dh			; add with carry hi byte
	daa				; decimal adjust
	mov	dh,al			; put it in dh
	ret


AD_cmd		db	58h		 
AD_address	dw	2B0h		
	
bcd_table:
		dw	0001h
		dw	0002h
		dw	0004h
		dw	0008h
		dw	0016h
		dw	0032h
		dw	0064h
		dw	0128h
		dw	0256h
		dw	0512H
		dw	1024h
		dw	2048h

help:		db	'Capture A/D data and save to a file',0Dh,0Ah
		db	'Usage:  Capture file...',0Dh,0Ah
		db	'Version 1.0 for JK microsystems SBC products'
		db	0Dh,0Ah,'$'

open_error:	db	'Error opening file',0Dh,0Ah,'$'
waiting:	db	'  Waiting for data...',0Dh,0Ah,'$'
converting:	db	'  Converting data...',0Dh,0Ah,'$'
writing:	db	'  Writing file...',0Dh,0Ah,'$'
write_error:	db	'Error writing file',0Dh,0Ah,'$'
close_error:	db	'Error closing file',0Dh,0Ah,'$'
crlf:		db	0Dh,0Ah,'$'
handle:		dw	?
filename:	db	20h	dup	(20h)	; buffer for filename
raw_buffer:	dw	800h dup (0)
raw_buffer_end:
ascii_buffer:	db	3000h dup (0)	
ascii_buffer_end:
		
code_seg 	ends

	end	start
